boolean[][] cellsNow; //2 dimensional array of bools
boolean[][] cellsFuture; //2 dimensional array of bools

// columns and rows are not really necessary, one could 
// use cells.length and cells[0].length instead, but makes 
// things clearer
int columns, rows;
int scale=5; //this means that the cells will be 5x5 pixels...

void setup(){

size(500,500);

frameRate(10); //10 frames per second
background(255); //draw background white
//stroke for showing the cells
stroke(200);
strokeWeight(0.5);

//we calculate the columns and rows so they fill the screen
//when we draw them...
columns=width/scale; 
rows=height/scale;

cellsNow=new boolean[columns][rows]; //in an array, booleans will be automatically set to false...
cellsFuture=new boolean[columns][rows];

//random seeding of the array
for(int i=0;i<columns;i++){//we go through all the columns
    for(int j=0;j<rows;j++){
       cellsNow[i][j]=random(0,100)<50;
    }
  }

//a glider...
 cellsNow[2][0]=true;

 cellsNow[0][1]=true;
 cellsNow[2][1]=true;

 cellsNow[1][2]=true;
 cellsNow[2][2]=true;

//another
// cellsNow[12][10]=true;

// cellsNow[10][11]=true;
// cellsNow[12][11]=true;

// cellsNow[11][12]=true;
// cellsNow[12][12]=true;


}


void draw(){
  background(255); //clean the background
  drawCells(); //draw

  tick(); //calculate future values
  update(); 
}


void drawCells(){
  for(int i=0;i<columns;i++){//we go through all the columns
    for(int j=0;j<rows;j++){
      if(cellsNow[i][j]==true){
        fill(0); //black if true
      }else{
        fill(255); //white if false
      }
      rect(i*scale,j*scale,scale,scale);
    }
  }
}


void tick(){
  for(int i=0;i<columns;i++){//we go through all the columns
    for(int j=0;j<rows;j++){
      int numNeigs= countNeighbours(i,j);
       cellsFuture[i][j]=applyRules(cellsNow[i][j], numNeigs);
    }
  }
}

int countNeighbours(int col,int row){
  //this is what is generally known as the "Moore neighborhood".
  //it comprises the 8 cells surrounding the one in col and row.
  //We have to be carefull with the cells in the borders (col==0,
  //col==colums-1, row==0 and row==rows-1). We decide to use a method 
  //similar to the"wrap around world" we have seen with the particles/agents.

  int totalNeighbours=0;

  for(int i=col-1;i<=col+1;i++){
    //wrap the column.
    int neigColumn=wrap(i,columns);
    for(int j=row-1;j<=row+1;j++){
      //wrap the row.
      int neigRow=wrap(j,rows);
      if(i==col&& j==row){
        continue;
      }
      // if this neighbour is "on" (true) we add it
      if(cellsNow[neigColumn][neigRow]==true){
        totalNeighbours++;
      }
    } //end of row loop
  } //end of column loop

  return totalNeighbours;
}

//this is the wrap function, as explained above.
int wrap(int index,int maxBound){
  int result;
  if(index<0){
     result=maxBound+index;
  }
  else if(index>=maxBound){
    result=index-maxBound;
  }
  else{
    result=index;
  }
  return result;
}

//These are the rules for the Game of Life, from Wikipedia
// Any live cell with fewer than two live neighbours dies, as if caused by underpopulation.
// Any live cell with two or three live neighbours lives on to the next generation.
// Any live cell with more than three live neighbours dies, as if by overpopulation.
// Any dead cell with exactly three live neighbours becomes a live cell, as if by reproduction.

boolean applyRules(boolean me, int numNeigs){

  boolean result=false;

  if(me==true){ //any alive cell...

    if(numNeigs<2){ //with fewer than two live neighbours
      result = false; //dies
    }
    else if(numNeigs==2 || numNeigs==3){//with two or three live neighbours
       result = true; //lives
    }else{//with more than three live neighbours
      result = false; //dies
    }
  }
  else{ //Any dead cell...

    if(numNeigs==3){ //with exactly three live neighbours
      result=true; //becomes a live cell
    }else{
      result=false;
    }
  }
  return result;
  
}


void update(){
  //copying all values.
  for(int i=0;i<columns;i++){//we go through all the columns
    for(int j=0;j<rows;j++){
      cellsNow[i][j]=cellsFuture[i][j];
    }
  }
}
